unit MainF;

//(*
//  BayesNet - Bayes Network Implementation
//
//  Sebastian Batton (s.batton@gmx.de)
//  Matthias Hupka (matthias-hupka@gmx.de)
//  Kevin Niehage (kevinniehage@gmx.de)
//  Johannes Weydt (johannes.weydt@gmx.de)
//
//  Duale Hochschule Mannheim - TAI06-ABC
//  Knowledge Based Systems - P.H. Vossen
//*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, BayesU, Menus, Grids, ComCtrls;

type
  TChosen = record
    Index : LongInt;
    Multi : Byte;
  end;
  TChosens = array of TChosen;

  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    FileMenuItem: TMenuItem;
    LoadCollectionMenuItem: TMenuItem;
    OpenDialog: TOpenDialog;
    HelpMenuItem: TMenuItem;
    PageControl: TPageControl;
    CollectionTabSheet: TTabSheet;
    CollectionStringGrid: TStringGrid;
    ValuesTabSheet: TTabSheet;
    ValueTypeListBox: TListBox;
    ValueTypeLabel: TLabel;
    ValueTrackBar: TTrackBar;
    MeaningLabel: TLabel;
    ValueLabel: TLabel;
    RatingLabel: TLabel;
    RatingTrackBar: TTrackBar;
    Null: TLabel;
    Eins: TLabel;
    Zwei: TLabel;
    CalculationTabSheet: TTabSheet;
    CalculationStringGrid: TStringGrid;
    procedure LoadCollectionMenuItemClick(Sender: TObject);
    procedure CollectionStringGridDrawCell(Sender: TObject; ACol,
      ARow: Integer; Rect: TRect; State: TGridDrawState);
    procedure ValueTypeListBoxClick(Sender: TObject);
    procedure ValueTrackBarChange(Sender: TObject);
    procedure RatingTrackBarChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure PageControlChange(Sender: TObject);
    procedure CalculationStringGridDrawCell(Sender: TObject; ACol,
      ARow: Integer; Rect: TRect; State: TGridDrawState);
    procedure HelpMenuItemClick(Sender: TObject);
  private
    { Private-Deklarationen }
    FCase          : TValueAssigns;
    FChosen        : TChosens;
    FCollection    : TValueCollection;
    FKnowledgeBase : TKnowledgeBase;
    FLoaded        : Boolean;
    FResult        : TResultLists;
  public
    { Public-Deklarationen }
  end;

var
  MainForm : TMainForm;

implementation

{$R *.dfm}

procedure TMainForm.LoadCollectionMenuItemClick(Sender: TObject);
var
  LIndexA : LongInt;
  LIndexB : LongInt;
  LIndexC : LongInt;
begin
  if OpenDialog.Execute then
  begin
    FLoaded := false;
    Caption := 'BayesNet';

    // reset everything
    SetLength(FCase,   0);
    SetLength(FChosen, 0);
    SetLength(FCollection.ResultSet,  0);
    SetLength(FCollection.ValueLists, 0);
    SetLength(FCollection.ValueSets,  0);
    SetLength(FKnowledgeBase.Entries, 0);
    SetLength(FKnowledgeBase.Results, 0);
    SetLength(FResult, 0);

    if LoadCollection(OpenDialog.FileName, FCollection) then
    begin
      if CheckCollection(FCollection) then
      begin
        FKnowledgeBase := GenerateKnowledgeBase(FCollection);

        // reset FChosen
        SetLength(FChosen, Length(FKnowledgeBase.Entries));
        for LIndexA := 0 to Pred(Length(FChosen)) do
        begin
          FChosen[LIndexA].Index := 0;
          FChosen[LIndexA].Multi := 1;
        end;

        // initialize string grid
        CollectionStringGrid.ColCount  := Length(FCollection.ValueSets) + 2;
        CollectionStringGrid.FixedCols := 0;
        CollectionStringGrid.RowCount  := Length(FCollection.ValueLists) + 1;
        CollectionStringGrid.FixedRows := 1;

        CollectionStringGrid.Cells[0, 0] := 'Entry';
        for LIndexA := 0 to Pred(Length(FCollection.ValueSets)) do
        begin
          CollectionStringGrid.Cells[LIndexA + 1, 0] := FCollection.ValueSets[LIndexA].Name;

          for LIndexB := 0 to Pred(Length(FCollection.ValueLists)) do
          begin
            CollectionStringGrid.Cells[0, LIndexB + 1] := IntToStr(LIndexB + 1);

            for LIndexC := 0 to Pred(Length(FCollection.ValueLists[LIndexB].Values)) do
            begin
              if (AnsiLowerCase(FCollection.ValueLists[LIndexB].Values[LIndexC].Name) = AnsiLowerCase(FCollection.ValueSets[LIndexA].Name)) then
              begin
                CollectionStringGrid.Cells[LIndexA + 1, LIndexB + 1] := AnsiLowerCase(FCollection.ValueLists[LIndexB].Values[LIndexC].Value);

                Break;
              end;
            end;

            CollectionStringGrid.Cells[Pred(CollectionStringGrid.ColCount), LIndexB + 1] := AnsiLowerCase(FCollection.ValueLists[LIndexB].Result);
          end;
        end;
        CollectionStringGrid.Cells[Pred(CollectionStringGrid.ColCount), 0] := 'Result';

        // initialize Value selection
        ValueTypeListBox.Clear;
        for LIndexA := 0 to Pred(Length(FCollection.ValueSets)) do
          ValueTypeListBox.Items.Add(FCollection.ValueSets[LIndexA].Name);
        ValueTypeListBox.ItemIndex := 0;
        ValueTypeListBoxClick(nil);

        FLoaded := true;
        Caption := Caption + ' - [' + ChangeFileExt(ExtractFileName(OpenDialog.FileName), '') + ']';
      end
      else
        MessageDlg('"' + ExtractFileName(OpenDialog.FileName) + '" does not contain a semantically correct collection.',
                   mtError, [mbOK], 0);
    end
    else
      MessageDlg('"' + ExtractFileName(OpenDialog.FileName) + '" is not a correct collection file.',
                 mtError, [mbOK], 0);

    PageControl.ActivePage := CollectionTabSheet;
    PageControl.Visible    := FLoaded;
  end;
end;

procedure TMainForm.CollectionStringGridDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
var
  LBold  : Boolean;
  LColor : TColor;
begin
  if (ACol = 0) then // entry
  begin
    LBold  := true;
    LColor := clBtnFace;
  end
  else
  begin
    if (ACol = Pred(CollectionStringGrid.ColCount)) then // result
    begin
      LBold  := true;
      LColor := RGB(255, 180, 180);
    end
    else
    begin // values
      LBold  := false;
      LColor := clWindow;
    end;
  end;

  CollectionStringGrid.Canvas.Brush.Color := LColor;
  CollectionStringGrid.Canvas.Font.Size   := 12;
  CollectionStringGrid.Canvas.Font.Style  := [];
  if (LBold) then
    CollectionStringGrid.Canvas.Font.Style  := [fsBold];

  CollectionStringGrid.Canvas.FillRect(Rect);
  CollectionStringGrid.Font.Color := clBlack;
  CollectionStringGrid.Canvas.TextOut(Rect.Left + 2, Rect.Top + 2, CollectionStringGrid.Cells[ACol, ARow]);
end;

procedure TMainForm.ValueTypeListBoxClick(Sender: TObject);
var
  LValue : LongInt;
begin
  LValue := FChosen[ValueTypeListBox.ItemIndex].Index;
  ValueTrackBar.Max := Pred(Length(FCollection.ValueSets[ValueTypeListBox.ItemIndex].Values));
  FChosen[ValueTypeListBox.ItemIndex].Index := LValue;

  RatingTrackBar.Position := FChosen[ValueTypeListBox.ItemIndex].Multi;
  ValueTrackBar.Position  := FChosen[ValueTypeListBox.ItemIndex].Index;

  MeaningLabel.Caption := AnsiLowerCase(FCollection.ValueSets[ValueTypeListBox.ItemIndex].Values[ValueTrackBar.Position]);
end;

procedure TMainForm.ValueTrackBarChange(Sender: TObject);
begin
  FChosen[ValueTypeListBox.ItemIndex].Index := ValueTrackBar.Position;

  MeaningLabel.Caption := AnsiLowerCase(FCollection.ValueSets[ValueTypeListBox.ItemIndex].Values[ValueTrackBar.Position]);
end;

procedure TMainForm.RatingTrackBarChange(Sender: TObject);
begin
  FChosen[ValueTypeListBox.ItemIndex].Multi := RatingTrackBar.Position;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  FLoaded := false;

  PageControl.ActivePage := CollectionTabSheet;
  PageControl.Visible    := FLoaded;
end;

procedure TMainForm.PageControlChange(Sender: TObject);
var
  LIndexA : LongInt;
  LIndexB : LongInt;
  LLeft   : LongInt;
  LResult : Extended;
  LTemp   : LongInt;
begin
  if (PageControl.ActivePage = CalculationTabSheet) then
  begin
    // generate case from chosen values
    SetLength(FCase, Length(FChosen));
    for LIndexA := 0 to Pred(Length(FChosen)) do
    begin
      FCase[LIndexA].Name  := FCollection.ValueSets[LIndexA].Name;
      FCase[LIndexA].Value := FCollection.ValueSets[LIndexA].Values[FChosen[LIndexA].Index];
    end;

    if SelectCaseResults(FKnowledgeBase, FCase, FResult) then
    begin
      LLeft := 0;
      for LIndexA := 0 to Pred(Length(FChosen)) do
      begin
        if (FChosen[LIndexA].Multi = 0) then
          Inc(LLeft);
      end;

      // initialize string grid
      CalculationStringGrid.ColCount  := Length(FCase) + 2 - LLeft;
      CalculationStringGrid.FixedCols := 0;
      CalculationStringGrid.RowCount  := Length(FResult) + 1;
      CalculationStringGrid.FixedRows := 1;

      CalculationStringGrid.Cells[0, 0] := 'Result';
      for LIndexA := 0 to Pred(Length(FResult)) do
      begin
        CalculationStringGrid.Cells[0, LIndexA + 1] := FResult[LIndexA].Result;

        LLeft := 0;
        LResult := 1;
        for LIndexB := 0 to Pred(Length(FCollection.ValueSets)) do
        begin
          if (FChosen[LIndexB].Multi > 0) then
          begin
            CalculationStringGrid.Cells[LIndexB + 1 - LLeft, 0] := FCollection.ValueSets[LIndexB].Name;

            LTemp := 0;
            if (FResult[LIndexA].Count > 0) then
            begin
              LTemp   := FChosen[LIndexB].Multi * FResult[LIndexA].Values[LIndexB].Count;
              LResult := LResult * LTemp / FResult[LIndexA].Count;
            end;

            CalculationStringGrid.Cells[LIndexB + 1 - LLeft, LIndexA + 1]   := IntToStr(FChosen[LIndexB].Multi) + ' * ' + IntToStr(FResult[LIndexA].Values[LIndexB].Count) + ' / ' + IntToStr(FResult[LIndexA].Count);
            CalculationStringGrid.Objects[LIndexB + 1 - LLeft, LIndexA + 1] := TObject(LTemp);
          end
          else
            Inc(LLeft);
        end;
        CalculationStringGrid.Cells[Pred(CalculationStringGrid.ColCount), LIndexA + 1] := FloatToStr(LResult);
      end;
      CalculationStringGrid.Cells[Pred(CalculationStringGrid.ColCount), 0] := 'Probability';
    end
    else
      MessageDlg('Error while selecting case results.',
                 mtError, [mbOK], 0);
  end;
end;

procedure TMainForm.CalculationStringGridDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
var
  LBold  : Boolean;
  LColor : TColor;
begin
  if (ACol = 0) then // result
  begin
    LBold  := true;
    LColor := RGB(255, 180, 180);
  end
  else
  begin
    if (ACol = Pred(CalculationStringGrid.ColCount)) then // probability
    begin
      LBold  := true;
      LColor := RGB(180, 255, 180);
    end
    else
    begin // values
      LBold  := false;

      if (ARow > 0) and (CalculationStringGrid.Objects[ACol, ARow] = TObject(0)) then
        LColor := RGB(255, 255, 180)
      else
        LColor := clWindow;
    end;
  end;

  CalculationStringGrid.Canvas.Brush.Color := LColor;
  CalculationStringGrid.Canvas.Font.Size   := 12;
  CalculationStringGrid.Canvas.Font.Style  := [];
  if (LBold) then
    CalculationStringGrid.Canvas.Font.Style  := [fsBold];

  CalculationStringGrid.Canvas.FillRect(Rect);
  CalculationStringGrid.Font.Color := clBlack;
  CalculationStringGrid.Canvas.TextOut(Rect.Left + 2, Rect.Top + 2, CalculationStringGrid.Cells[ACol, ARow]);
end;

procedure TMainForm.HelpMenuItemClick(Sender: TObject);
begin
  MessageDlg('//(*' + #13#10 +
             '//  BayesNet - Bayes Network Implementation' + #13#10 +
             '//' + #13#10 +
             '//  Sebastian Batton (s.batton@gmx.de)' + #13#10 +
             '//  Matthias Hupka (matthias-hupka@gmx.de)' + #13#10 +
             '//  Kevin Niehage (kevinniehage@gmx.de)' + #13#10 +
             '//  Johannes Weydt (johannes.weydt@gmx.de)' + #13#10 +
             '//' + #13#10 +
             '//  Duale Hochschule Mannheim - TAI06-ABC' + #13#10 +
             '//  Knowledge Based Systems - P.H. Vossen' + #13#10 +
             '//*)', mtInformation, [mbOK], 0);
end;

end.
 